@@ -17,6 +17,9 @@ MIN_RETRY_INTERVAL = 2 # 2 seconds
 MAX_RETRY_INTERVAL = 300 # 5 minutes
 MAX_RETRY_ELAPSED_TIME = 43200 # 12 hours
 
+# Break up reports_email_activity batches to iterate over chunks
+EMAIL_ACTIVITY_BATCH_SIZE = 100
+
 class BatchExpiredError(Exception):
     pass
 
@@ -185,7 +188,7 @@ def sync_stream(client,
 
         if endpoint_config.get('store_ids'):
             id_bag[stream_name] = stream_ids
-        
+
         children = endpoint_config.get('children')
         if children:
             for child_stream_name, child_endpoint_config in children.items():
@@ -296,17 +299,7 @@ def stream_email_activity(client, catalog, state, archive_url):
                 file = tar.next()
     return failed_campaign_ids
 
-def sync_email_activity(client, catalog, state, start_date, campaign_ids):
-    batch_id = get_bookmark(state, ['reports_email_activity_last_run_id'], None)
-
-    if batch_id:
-        try:
-            get_batch_info(client, batch_id)
-        except BatchExpiredError:
-            LOGGER.info('reports_email_activity - Previous run from state expired: {}'.format(
-                batch_id))
-            batch_id = None
-
+def sync_email_activity(client, catalog, state, start_date, campaign_ids, batch_id=None):
     if batch_id:
         LOGGER.info('reports_email_activity - Picking up previous run: {}'.format(batch_id))
     else:
@@ -347,7 +340,8 @@ def sync_email_activity(client, catalog, state, start_date, campaign_ids):
                                                 catalog,
                                                 state,
                                                 data['response_body_url'])
-    LOGGER.warning("reports_email_activity - operations failed for campaign_ids: %s", failed_campaign_ids)
+    if failed_campaign_ids:
+        LOGGER.warning("reports_email_activity - operations failed for campaign_ids: %s", failed_campaign_ids)
 
     write_activity_batch_bookmark(state, None)
 
@@ -372,6 +366,59 @@ def should_sync_stream(streams_to_sync, dependants, stream_name):
             return True, should_persist
     return False, should_persist
 
+def chunk_campaigns(sorted_campaigns, chunk_bookmark):
+    chunk_start = chunk_bookmark * EMAIL_ACTIVITY_BATCH_SIZE
+    chunk_end = chunk_start + EMAIL_ACTIVITY_BATCH_SIZE
+
+    if chunk_bookmark > 0:
+        LOGGER.info("reports_email_activity - Resuming requests starting at campaign_id %s (index %s) in chunks of %s",
+                    sorted_campaigns[chunk_start],
+                    chunk_start,
+                    EMAIL_ACTIVITY_BATCH_SIZE)
+
+    done = False
+    while not done:
+        current_chunk = sorted_campaigns[chunk_start:chunk_end]
+        done = len(current_chunk) == 0
+        if not done:
+            end_index = min(chunk_end, len(sorted_campaigns))
+            LOGGER.info("reports_email_activity - Will request for campaign_ids from %s to %s (index %s to %s)",
+                        sorted_campaigns[chunk_start],
+                        sorted_campaigns[end_index - 1],
+                        chunk_start,
+                        end_index - 1)
+            yield current_chunk
+        chunk_start = chunk_end
+        chunk_end += EMAIL_ACTIVITY_BATCH_SIZE
+
+def write_email_activity_chunk_bookmark(state, current_bookmark, current_index, sorted_campaigns):
+    # Bookmark next chunk because the current chunk will be saved in batch_id
+    # Index is relative to current bookmark
+    next_chunk = current_bookmark + current_index + 1
+    if next_chunk * EMAIL_ACTIVITY_BATCH_SIZE < len(sorted_campaigns):
+        write_bookmark(state, ['reports_email_activity_next_chunk'], next_chunk)
+    else:
+        write_bookmark(state, ['reports_email_activity_next_chunk'], 0)
+
+def check_and_resume_email_activity_batch(client, catalog, state, start_date):
+    batch_id = get_bookmark(state, ['reports_email_activity_last_run_id'], None)
+
+    if batch_id:
+        try:
+            data = get_batch_info(client, batch_id)
+            if not data['response_body_url']:
+                LOGGER.info('reports_email_activity - Previous run from state ({}) is empty, retrying.'.format(
+                    batch_id))
+                return
+        except BatchExpiredError:
+            LOGGER.info('reports_email_activity - Previous run from state expired: {}'.format(
+                batch_id))
+            return
+
+        # Resume from bookmarked job_id, then if completed, issue a new batch for processing.
+        campaigns = [] # Don't need a list of campaigns if resuming
+        sync_email_activity(client, catalog, state, start_date, campaigns, batch_id)
+
 ## TODO: is current_stream being updated?
 
 def sync(client, catalog, state, start_date):
@@ -448,4 +495,14 @@ def sync(client, catalog, state, start_date):
                                                        'reports_email_activity')
     campaign_ids = id_bag.get('campaigns')
     if should_stream and campaign_ids:
-        sync_email_activity(client, catalog, state, start_date, campaign_ids)
+        # Resume previous batch, if necessary
+        check_and_resume_email_activity_batch(client, catalog, state, start_date)
+        # Chunk batch_ids, bookmarking the chunk number
+        sorted_campaigns = sorted(campaign_ids)
+        chunk_bookmark = int(get_bookmark(state, ['reports_email_activity_next_chunk'], 0))
+        for i, campaign_chunk in enumerate(chunk_campaigns(sorted_campaigns, chunk_bookmark)):
+            write_email_activity_chunk_bookmark(state, chunk_bookmark, i, sorted_campaigns)
+            sync_email_activity(client, catalog, state, start_date, campaign_chunk)
+
+        # Start from the beginning next time
+        state = write_bookmark(state, ['reports_email_activity_next_chunk'], 0)
